package
{
	import org.flixel.*;

	public class Player extends FlxSprite
	{
		[Embed(source="/assets/spaceman.png")] protected var ImgSpaceman:Class;
		
		protected var _jumpPower:int;
		protected var _aim:uint;
		protected var _restart:Number;
		
		//This is the player object class.  Most of the comments I would put in here
		//would be near duplicates of the Enemy class, so if you're confused at all
		//I'd recommend checking that out for some ideas!
		public function Player(X:int,Y:int)
		{
			super(X,Y);
			loadGraphic(ImgSpaceman,true,true,8);
			_restart = 0;
			
			//bounding box tweaks
			width = 6;
			height = 7;
			offset.x = 1;
			offset.y = 1;
			
			//basic player physics
			var runSpeed:uint = 80;
			drag.x = runSpeed*8;
			acceleration.y = 420;
			_jumpPower = 200;
			maxVelocity.x = runSpeed;
			maxVelocity.y = _jumpPower;
			
			//animations
			addAnimation("idle", [0]);
			addAnimation("run", [1, 2, 3, 0], 12);
			addAnimation("jump", [4]);
			addAnimation("idle_up", [5]);
			addAnimation("run_up", [6, 7, 8, 5], 12);
			addAnimation("jump_up", [9]);
			addAnimation("jump_down", [10]);
			
		}
		
		override public function destroy():void
		{
			super.destroy();
		}
		
		override public function update():void
		{
			//game restart timer
			if(!alive)
			{
				_restart += FlxG.elapsed;
				if(_restart > 2)
					FlxG.resetState();
				return;
			}
			
			//MOVEMENT
			acceleration.x = 0;
			if(FlxG.keys.LEFT)
			{
				facing = LEFT;
				acceleration.x -= drag.x;
			}
			else if(FlxG.keys.RIGHT)
			{
				facing = RIGHT;
				acceleration.x += drag.x;
			}
			if(FlxG.keys.justPressed("X") && !velocity.y)
			{
				velocity.y = -_jumpPower;
			}
			
			//AIMING
			if(FlxG.keys.UP)
				_aim = UP;
			else if(FlxG.keys.DOWN && velocity.y)
				_aim = DOWN;
			else
				_aim = facing;
			
			//ANIMATION
			if(velocity.y != 0)
			{
				if(_aim == UP) play("jump_up");
				else if(_aim == DOWN) play("jump_down");
				else play("jump");
			}
			else if(velocity.x == 0)
			{
				if(_aim == UP) play("idle_up");
				else play("idle");
			}
			else
			{
				if(_aim == UP) play("run_up");
				else play("run");
			}
		}
		
		override public function hurt(Damage:Number):void
		{
			Damage = 0;
			if(flickering)
				return;
			flicker(1.3);
			super.hurt(Damage);
		}
		
		override public function kill():void
		{
			if(!alive)
				return;
			solid = false;
			super.kill();
			flicker(0);
			exists = true;
			visible = false;
			velocity.make();
			acceleration.make();
			FlxG.camera.shake(0.005,0.35);
			FlxG.camera.flash(0xffd8eba2,0.35);
		}
	}
}